
USE work.STD_LOGIC.ALL;
USE WORK.ALL;
entity inverter is
    generic (gdelay: time:= 3 ps;
             MaxStrength0 : t_strength := 'R';
             MaxStrength1 : t_strength := 'R');
    port( i: in t_wlogic; o: inout t_wlogic);
end inverter;
--
architecture inverter_behavior of inverter is
    component pfet 
          generic (gdelay : time := 3 ps;
                   MaxStrength: t_strength := 'R');
          port( g: in t_wlogic; src,drn: inout t_wlogic); 
    end component;
    component nfet 
          generic (gdelay : time := 3 ps;
                   MaxStrength: t_strength := 'R');
          port( g: in t_wlogic; src,drn: inout t_wlogic); 
    end component;
    signal vdd: t_wlogic:= F1;
    signal gnd: t_wlogic:= F0;
begin
    i0: pfet generic map ( gdelay, MaxStrength0) port map( i, vdd, o) ;
    i1: nfet generic map ( gdelay, MaxStrength1) port map( i, gnd, o );
    vdd <= F1;
    gnd <= F0;
end inverter_behavior;

--
--------------------------------------------------
-- pswitch: unidirectional gate with two strengths
--------------------------------------------------
--
USE work.STD_LOGIC.ALL;
USE WORK.ALL;
entity pswitch is
    generic (gdelay : time := 3 ps; 
             MaxStrength0 : t_strength := 'R';
             MaxStrength1 : t_strength := 'R');
    port( g,src: in t_wlogic; drn: inout t_wlogic);
         -- Src and Drn are symmetric.
end pswitch;
--
architecture pswitch_behavior of pswitch is
    component pfet 
          generic (gdelay : time := 3 ps;
                   MaxStrength: t_strength:= 'R');
          port( g: in t_wlogic; src,drn: inout t_wlogic); 
    end component;
    component inverter 
          generic (gdelay : time := 3 ps;
                   MaxStrength0: t_strength:= 'R';
                   MaxStrength1: t_strength:= 'R');
          port( i: in t_wlogic; o: inout t_wlogic); 
    end component;

    signal s: t_wlogic;
begin
    i0: inverter generic map ( gdelay, MaxStrength0, MaxStrength1) 
                 port map( src, s);
    i1: pfet generic map ( gdelay, 'F') port map(g, s, drn);
end pswitch_behavior;

