
USE WORK.STD_LOGIC.ALL;
USE work.ALL;

entity mem1 is
    port( H: inout t_wlogic; A,B,C,D: in t_wlogic);
end mem1;

architecture mem1_sw of mem1 is
    component nfet 
          generic (gdelay : time := 3 ps; MaxStrength : t_strength := 'R');
          port( g: in t_wlogic; src,drn: inout t_wlogic); end component;
    component pswitch 
          generic (gdelay : time := 3 ps; 
                   MaxStrength0 : t_strength := 'F';
                   MaxStrength1 : t_strength := 'R'
                   );
          port( g, src: in t_wlogic; drn: inout t_wlogic); end component;
    component inverter 
          generic (gdelay: time := 3 ps;
                   MaxStrength0 : t_strength := 'F';
                   MaxStrength1 : t_strength := 'R'
                   );
          port( i: in t_wlogic; o: inout t_wlogic); end component;

    signal E, F, G:   t_wlogic;

begin
-- The commented version works, but it breaks the general rule that 
-- no bxfr should have MAXSTRENGTH = `F`. Strength 'F' is reserved
-- for vdd and gnd. An optimization is performed in the bxfr architecture
-- so that no attempt should be made to modify the value of a signal 
-- having a value of strength 'F', i.e. vdd or gnd. Under certain 
-- circumstances the disconnection/reconnection algorithm of the bxfr
-- may not work if the optimization is implemented and a signal 
-- other than vdd or gnd has the strength 'F'. The result may be that
-- that signal will not change its value when it should.
-- 
--    U1: pswitch
--          generic map(1 ps, 'F', 'R') port map( B, A, G );
--    U2: nfet generic map(1 ps, 'R') port map( C, G, H );
--    U3: nfet generic map(1 ps, 'R') port map( D, G, E );
--    U4: inverter generic map(3 ps, 'F', 'R') port map( E, F );
--    U5: inverter generic map(3 ps, 'W', 'W') port map( F, E );
    U1: pswitch
          generic map(1 ps, 'R', 'R') port map( B, A, G );
    U2: nfet generic map(1 ps, 'R') port map( C, G, H );
    U3: nfet generic map(1 ps, 'R') port map( D, G, E );
    U4: inverter generic map(3 ps, 'R', 'R') port map( E, F );
    U5: inverter generic map(3 ps, 'W', 'W') port map( F, E );

end mem1_sw;
