
----------------------------------------------------------------
----------------------------------------------------------------
--
--  Copyright 1989, Andy Tsay and Alec Stanculescu
--
--  Note: The code in this file has been developed 
--        by Andy Tsay and Alec Stanculescu.
--
--        Vantage Analysis Systems, Inc. has graciously
--        placed the Std_Logic Package in the public domain.
--
--        The only restrictions in using this code are:
--        1. Observe the export regulations of the United States
--           of America. When in doubt check with the Department
--           of Commerce, with Vantage Analysis Systems, Inc, or with 
--           Alec Stanculescu (Fax 415 578 0260).
--        2. Do not separate the Copyright notice nor this note 
--           from the code.
--        3. If you modify the code, give proper credit to this 
--           version by acknowledging: Vantage Analysis Systems,
--           Andy Tsay, and Alec Stanculescu.
--
USE WORK.STD_LOGIC.ALL;
USE work.ALL;

entity ram1 is
    port( d, db: inout t_wlogic; rwd, rwdb: in t_wlogic);
end ram1;

architecture ram1_sw of ram1 is
    component nfet_type 
          generic (gdelay : time := 3 ps; MaxStrength : t_strength := 'R');
          port( g: in t_wlogic; src,drn: inout t_wlogic); end component;
    component pfet_type 
          generic (gdelay : time := 3 ps; MaxStrength : t_strength := 'R');
          port( g: in t_wlogic; src,drn: inout t_wlogic); end component;
    signal vdd : t_wlogic := F1;
    signal vss : t_wlogic := F0;
    signal s1:   t_wlogic;
    signal s2:   t_wlogic;
begin
    i1: nfet_type generic map(1 ps, 'R') port map( rwd, d, s1 );
    i2: nfet_type generic map(1 ps, 'W') port map( s1, s2, vss );
    i3: pfet_type generic map(1 ps, 'W') port map( s1, s2, vdd );
    i4: pfet_type generic map(1 ps, 'W') port map( s2, s1, vdd );
    i5: nfet_type generic map(1 ps, 'W') port map( s2, s1, vss );
    i6: nfet_type generic map(1 ps, 'R') port map( rwdb, db, s2 );
    vss <= F0;
    vdd <= F1;
end ram1_sw;

use work.all;
configuration ram1_conf of ram1 is
    for ram1_sw
        for all: pfet_type use configuration work.pfet_conf
                    generic map(gdelay, MaxStrength) port map (g,src,drn);
        end for;
        for all: nfet_type use configuration work.nfet_conf
                    generic map(gdelay, MaxStrength) port map (g,src,drn);
        end for;
    end for;
end ram1_conf;

USE WORK.STD_LOGIC.ALL;
USE work.ALL;

entity ram1_top is
end ram1_top;

architecture ram1_top_sw of ram1_top is
    component ram1_type 
        port( d, db: inout t_wlogic; rwd, rwdb: in t_wlogic);
    end component;
    signal sd,sdb,rw: t_wlogic := F0;
begin
    i0: ram1_type port map(sd,sdb,rw,rw);
    sdb <= d;
    rw <= f1, f0 after 10 ns, f1 after 20 ns, f0 after 30 ns, f1 after 40 ns;
    sd <= r0, r1 after 1 ns, z1 after 2 ns, r0 after 7 ns, z0 after 8 ns,
          r1 after 9 ns, r0 after 11 ns, r1 after 15 ns, z1 after 17 ns;
end ram1_top_sw;

use work.all;
configuration ram1_top_conf of ram1_top is
    for ram1_top_sw
        for all: ram1_type use configuration work.ram1_conf;
        end for;
    end for;
end ram1_top_conf;
