USE work.analog_data_structure.all;
PACKAGE analog_active_behaviors IS
--
-- function for VCO - returns the total vco cap voltage
--
   FUNCTION vco_vcap  ( vcap	  : analog_signal; --v
			vin	  : analog_signal; --v
			slope	  : real;	 --ghz/v
			intercept : real;	 --ghz
			vhi	  : real;	 --v
			vlo	  : real;	 --v
			vco_out   : bit 	)
				  RETURN real;
--
--
-- function for VCO - returns the total vco cap voltage
--
   FUNCTION vco_vcapf ( vcap	  : analog_signal; --v
			vin	  : analog_signal; --v
			slope	  : real;	 --ghz/v
			intercept : real;	 --ghz
			vhi	  : real;	 --v
			vlo	  : real;	 --v
			vco_out   : bit )
				  RETURN real;
--
--
-- function for VGA - returns a gain for the given voltage
--
   FUNCTION variable_gain ( vin : analog_signal;
			    slope     : real; --ghz/v
			    intercept : analog_vv_gain)
				      RETURN analog_v_level;
--
END analog_active_behaviors;


PACKAGE BODY analog_active_behaviors IS
--
--
-- function for VCO - returns the total vco cap voltage
--
   FUNCTION vco_vcap  ( vcap	  : analog_signal; --v
			vin	  : analog_signal; --v
			slope	  : real;	 --ghz/v
			intercept : real;	 --ghz
			vhi	  : real;	 --v
			vlo	  : real;	 --v
			vco_out   : bit 	)
				  RETURN real IS

     VARIABLE	ptime	     :	real; --ns
     VARIABLE	v_delta      :	real; --v
     VARIABLE	v_cap_tot    :	real;
     VARIABLE	pfreq	     :	real; --ghz

    BEGIN
      pfreq   := ( slope * vin.val ) + intercept;  --desired frequency
      ptime   := ( 0.5/pfreq ); 		   --half period time

      v_delta := ( (vhi-vlo)/ptime ) * analog_time_delta_real; -- + delta v

      IF (vco_out = '1') THEN
	 v_delta := -1.0 * v_delta;		   --ramp down not up
      END IF;
      v_cap_tot := vcap.val + v_delta;	       -- projected vcap
      IF (v_cap_tot > vhi ) THEN
	 v_cap_tot := vcap.val-(v_delta - ( 2.0*(vhi - vcap.val) ));
      END IF;
      IF (v_cap_tot < vlo ) THEN
	 v_cap_tot := vcap.val-(v_delta - ( 2.0*(vlo - vcap.val) ));
      END IF;
      RETURN v_cap_tot;
    END vco_vcap;
--
--
--
-- function for VCO - returns the total vco cap voltage
--
   FUNCTION vco_vcapf ( vcap	  : analog_signal; --v
			vin	  : analog_signal; --v
			slope	  : real;	 --ghz/v
			intercept : real;	 --ghz
			vhi	  : real;	 --v
			vlo	  : real;	 --v
			vco_out   : bit )
				  RETURN real IS

     VARIABLE	ptime	     :	real; --ns
     VARIABLE	v_delta      :	real; --v
     VARIABLE	v_cap_flg    :	real;
     VARIABLE	pfreq	     :	real; --ghz

    BEGIN
      pfreq   := ( slope * vin.val ) + intercept;  --desired frequency
      ptime   := ( 0.5/pfreq ); 		   --half period time

      v_delta := ( (vhi-vlo)/ptime ) * analog_time_delta_real; -- + delta v

      IF (vco_out = '1') THEN
	 v_delta := -1.0 * v_delta;		   --ramp down not up
      END IF;
      v_cap_flg := vcap.val + v_delta;	       -- projected vcap
      IF (v_cap_flg > vhi ) THEN
	 v_cap_flg := (vhi + 0.1);
      END IF;
      IF (v_cap_flg < vlo ) THEN
	 v_cap_flg := (vlo - 0.1);
      END IF;
      RETURN v_cap_flg;
    END vco_vcapf;
--
--
-- function for VGA - returns a gain for the given voltage
--
   FUNCTION variable_gain ( vin : analog_signal;
			    slope     : real; --ghz/v
			    intercept : analog_vv_gain)
				      RETURN analog_v_level IS

     VARIABLE	gain	     :	analog_v_level;

    BEGIN

      gain.val := (slope * vin.val) + intercept.val;

      IF (gain.val < 0.001) THEN
	 gain.val :=  0.0;
      END IF;

      IF (gain.val > 1000.0) THEN
	 gain.val := 1000.0;
      END IF;

      RETURN gain;
    END variable_gain;
--
END analog_active_behaviors;
