USE work.analog_data_structure.all;
PACKAGE analog_passive_behaviors IS
--
-- function for RC - Low Pass
--
 FUNCTION rc ( r	    : analog_component;
	       c	    : analog_component;
	       pred1_vout   : analog_signal;
	       vin	    : analog_signal;
	       clamp_lo     : analog_v_level;
	       clamp_hi     : analog_v_level)
			    RETURN analog_signal;
--
--
-- function for Cap filter- integrator	    vout-C-gnd
--
   FUNCTION capf  ( c	      : analog_component;
		    iin       : analog_signal;
		    pred1_vout: analog_signal )
			      RETURN analog_signal;
--
--
-- function for VFO filter - (2 poles 1 zero)	     vout-C2-C1-gnd
--							    -R1-gnd
--
   FUNCTION vfof(  r1		: analog_component;
		   c1		: analog_component;
		   c2		: analog_component;
		   pred1_iin	: analog_signal;
		   iin		: analog_signal;
		   pred2_vout	: analog_signal;
		   pred1_vout	: analog_signal )
				RETURN analog_signal;
--
END analog_passive_behaviors;


PACKAGE BODY analog_passive_behaviors IS
--
-- function for RC - Low Pass
--
 FUNCTION rc ( r	    : analog_component;
	       c	    : analog_component;
	       pred1_vout   : analog_signal;
	       vin	    : analog_signal;
	       clamp_lo     : analog_v_level;
	       clamp_hi     : analog_v_level)
			    RETURN analog_signal IS

  VARIABLE   tau	 :  real;
  VARIABLE   vout	 :  analog_signal;

 BEGIN
    tau       := ( r.val * c.val);
    vout.val  := ( (pred1_vout.val * tau)    +
		   (vin.val * analog_time_delta_real) )
		     /(tau  + analog_time_delta_real);

-- clamp limits
  IF	(vout.val > clamp_hi.val) THEN vout.val := clamp_hi.val;
  ELSIF (vout.val < clamp_lo.val) THEN vout.val := clamp_lo.val;
  END IF;

  vout.kind := v;
  RETURN vout;

 END rc;
--
--
-- function for Cap filter- integrator	    vout-C-gnd
--
   FUNCTION capf  ( c	      : analog_component;
		    iin       : analog_signal;
		    pred1_vout: analog_signal )
			      RETURN analog_signal IS

    VARIABLE   vout  :	analog_signal;

    BEGIN
	  vout.val := ( (iin.val/c.val)*analog_time_delta_real +
					      pred1_vout.val );

	  vout.kind := v;
	  RETURN vout;
    END capf;
--
--
-- function for VFO filter - (2 poles 1 zero)	     vout-C2-C1-gnd
--							    -R1-gnd
--
   FUNCTION vfof(  r1		: analog_component;
		   c1		: analog_component;
		   c2		: analog_component;
		   pred1_iin	: analog_signal;
		   iin		: analog_signal;
		   pred2_vout	: analog_signal;
		   pred1_vout	: analog_signal )
				RETURN analog_signal IS

     VARIABLE	num1, num2, num3, num4 :  real;
     VARIABLE	k1  , k2  , k3	       :  real;
     VARIABLE	denom		       :  real;
     VARIABLE	vout		       :  analog_signal;

    BEGIN
	  k1   := ( r1.val * c1.val * c2.val );
	  k2   := ( c2.val );
	  k3   := ( r1.val * (c1.val + c2.val) );

	  num1 := ( (analog_time_delta_real*analog_time_delta_real +
		      k3*analog_time_delta_real)* iin.val );
	  num2 := ( (k3*analog_time_delta_real) * pred1_iin.val );
	  num3 := ( (2.0*k1) + (k2*analog_time_delta_real) ) *
		    pred1_vout.val;
	  num4 := ( (k1) * pred2_vout.val );

	  denom:= ( (k1) + (k2*analog_time_delta_real) );

	  vout.val:= ( (num1 - num2 + num3 - num4)/denom );
	  vout.kind := v;

	  RETURN vout;
    END vfof;
--
--
END analog_passive_behaviors;
