PACKAGE summing_bus_res_function IS
--
-- Declare subtype of a real for bus resolution function that is
-- an unconstrained array.  The size of summing_val array equals the
-- number of signal assignment statements assigned to the common signal
-- that is currently calling the function.
--
   TYPE summing_val IS ARRAY ( integer RANGE<> ) OF real;
--
-- The function for bus resolution sums the currents or voltages at the
-- common signal (dot), hence, if one of the components is disabled its
-- signal assignment statement driving the signal should be assigned 0.0
-- in the behavior
--
   FUNCTION summing_brf_val ( data_array: summing_val )RETURN real;
--
   SUBTYPE brf_real IS summing_brf_val real;
--
END summing_bus_res_function;


PACKAGE BODY summing_bus_res_function IS
--

   FUNCTION summing_brf_val ( data_array: summing_val )RETURN real IS

     VARIABLE	i	    :  integer;
     VARIABLE	result	    :  real := 0.0;

    BEGIN
	  FOR i IN data_array'RANGE
	  LOOP
	     result := result + data_array(i);
	  END LOOP;

	  RETURN result;
    END summing_brf_val;
--
END summing_bus_res_function;



