USE work.tline_data_structure.all;
USE work.tline_active_behaviors.all;

-- interface
ENTITY linear_drv IS
    GENERIC ( hi      : real	   :=  5.0  ; -- volts
	      low     : real	   :=  0.0  ; -- volts
	      d_kind  : sig_mode   :=  v    ;
	      r_out   : real	   :=  0.080; -- kohms
	      t_delay : time	   :=  1000 ps	  );
    PORT    ( drv_in  : IN     bit;
	      drv_out : INOUT  tline_node := (volt  =>0.0, g0	 =>g_min,
					      volt_fe=>0.0, cur_fe=>0.0));

BEGIN

       ASSERT( r_out > 0.0 )
	  REPORT "ERROR: linear_drv Rout must be > 0.0"
	  SEVERITY failure;

END linear_drv;

-- body
ARCHITECTURE linear_drv_behavior OF linear_drv IS
     SIGNAL listen_toggle : boolean := false;

     SIGNAL drv_in_delay  : bit := '0';

    BEGIN
     drv_out.g0 <= ( 1.0/(r_out + r_min) );
     drv_in_delay <= drv_in AFTER t_delay;


P1:  PROCESS(drv_out.volt, drv_in_delay, listen_toggle)

       VARIABLE this_driver   : boolean := false;

     BEGIN
	IF ( NOT this_driver ) THEN
	   drv_out.volt <= drv_linear(drv_in_delay, r_out, drv_out,
				      low, hi, d_kind);
	   this_driver := TRUE;
	   listen_toggle <= NOT listen_toggle;
	ELSE
	   this_driver := FALSE;
	END IF;
     END PROCESS p1;
--
END linear_drv_behavior;
