USE work.tline_data_structure.all;
USE work.tline_active_behaviors.all;

-- interface of transmission line model
ENTITY tline IS
    GENERIC ( r_char  : real   :=  0.080; -- kohms
	      t_flight: time   := 10000 ps	 );
    PORT    (  node1  : INOUT	tline_node := (volt=>0.0, g0=>g_min,
					       volt_fe=>0.0, cur_fe=>0.0);
	       node2  : INOUT	tline_node := (volt=>0.0, g0=>g_min,
					       volt_fe=>0.0, cur_fe=>0.0));

BEGIN

    ASSERT( r_char > 0.0 )
       REPORT "ERROR: tline R0 must be > 0.0"
       SEVERITY failure;

END tline;

-- body of transmission line model
ARCHITECTURE tline_behavior OF tline IS
      SIGNAL int1, int2 : tline_traveler := ( volt => 0.0 );
      SIGNAL br1, br2 : tline_branch := ( cur => 0.0 );
    BEGIN
     node1.g0 <= ( 1.0/(r_char + r_min) );
     node2.g0 <= ( 1.0/(r_char + r_min) );

P1:  PROCESS(node1.volt)
     BEGIN
       node2.volt <= TRANSPORT transmit_volt (node1, int1, node2,r_char)
							 AFTER t_flight;
       node2.volt_fe <= TRANSPORT transmit_vfe (node1)
							 AFTER t_flight;
       node2.cur_fe  <= TRANSPORT update_current (node1, int1, r_char)
							 AFTER t_flight;
       int2.volt     <= TRANSPORT propagate (node1, int1)
							 AFTER t_flight;
       br1.cur	     <= TRANSPORT update_current (node1, int1, r_char);

     END PROCESS p1;
P2:  PROCESS(node2.volt)
     BEGIN
       node1.volt <= TRANSPORT transmit_volt (node2, int2, node1,r_char)
							 AFTER t_flight;
       node1.volt_fe <= TRANSPORT transmit_vfe (node2)
							 AFTER t_flight;
       node1.cur_fe   <= TRANSPORT update_current (node2, int2, r_char)
							 AFTER t_flight;
       int1.volt      <= TRANSPORT propagate (node2, int2)
							 AFTER t_flight;
       br2.cur	      <= TRANSPORT update_current (node2, int2, r_char);

     END PROCESS p2;
--
END tline_behavior;
