USE work.tline_data_structure.all;
USE work.tline_active_behaviors.all;

-- interface
ENTITY receiver IS
    GENERIC ( v_sw_hi  : real	:=  1.0  ; -- volts
	      v_sw_low : real	:=  1.0  ; -- volts
	      r_input  : real	:=  0.080; -- kohms
	      t_delay  : time	:=  1 ns	);
    PORT    ( rec_in   : INOUT	tline_node := (volt=>0.0, g0=>g_min,
					       volt_fe=>0.0, cur_fe=>0.0);
	      rec_out: BUFFER bit	);
 BEGIN

     ASSERT( r_input > 0.0 )
	REPORT "ERROR: receiver Rin must be > 0.0"
	SEVERITY failure;

END receiver;

-- body
ARCHITECTURE receiver_behavior OF receiver IS
    BEGIN
     rec_in.g0	<= ( 1.0 / (r_input + r_min) );

     rec_out <= '1' AFTER t_delay WHEN (rec_in.volt > v_sw_hi ) ELSE
		'0' AFTER t_delay WHEN (rec_in.volt < v_sw_low) ELSE
	    rec_out AFTER t_delay;

--
END receiver_behavior;
