USE work.tline_data_structure.all;
USE work.tline_active_behaviors.all;

-- interface
ENTITY fet_od_drv IS
    GENERIC ( gm      : real   := 5.0  ; -- a/(v*v)
	      wl      : real   := 0.0  ;
	      vt      : real   := 0.0  ; -- volts
	      vgs     : real   := 0.0  ; -- volts
	      t_delay : time   := 100 ns);
    PORT    ( din     : IN  bit;
	      dout    : INOUT  tline_node := (volt=>0.0, g0=>g_min,
					    volt_fe=>0.0, cur_fe=>0.0));

BEGIN

    ASSERT( (gm > 0.0) AND (wl > 0.0) AND (vt > 0.0) AND (vgs > 0.0)  )
       REPORT "ERROR: tline_fet_drv generics must be > 0.0"
       SEVERITY failure;

END fet_od_drv;

-- body
ARCHITECTURE fet_od_drv_behavior OF fet_od_drv IS
    BEGIN
     dout.g0 <= g_max; --short it so tline drives 0 volts

P1:  PROCESS(dout.volt_fe, din)
     BEGIN
	   dout.volt <= drive_fet(din, dout, gm, wl, vt, vgs);

     END PROCESS p1;
--
END fet_od_drv_behavior;
