USE work.analog_data_structure.all;
USE work.analog_passive_behaviors.all;
USE work.analog_active_behaviors.all;

-- interface
ENTITY vg_amplifier  IS
    GENERIC ( slope	: real;
	      intcpt	: analog_vv_gain;
	      vcmo	: analog_v_level;
	      vout_min	: analog_v_level;
	      vout_max	: analog_v_level;
	      r 	: analog_component;
	      c 	: analog_component);
    PORT    ( vinp    : IN     analog_signal;
	      vinn    : IN     analog_signal;
	      vgain   : IN     analog_signal;
	      vout    : BUFFER analog_signal := (val => 0.0, kind => v));

BEGIN
    ASSERT( (r.val > 0.0) AND (c.val > 0.0) )
       REPORT "ERROR: R and C must be > 0"
       SEVERITY failure;

    ASSERT( vout_max.val > vout_min.val )
       REPORT "ERROR: vout_max must be > vout_min"
       SEVERITY failure;
END vg_amplifier;

-- body
ARCHITECTURE vg_amplifier_behavior OF vg_amplifier IS
      SIGNAL ac_in, ideal_vout, pvout : analog_signal :=
	     (val => 0.0, kind => v );

      SIGNAL gain    : analog_v_level :=
	     (val => 0.0 );

-- instansiate
    BEGIN
      -- Ideal Amplifier
      gain  <= variable_gain( vgain, slope, intcpt );
      ac_in.val  <= (vinp.val - vinn.val) * 0.5;
      ideal_vout.val <= vcmo.val + ( ac_in.val *  gain.val );

      -- Single Pole
      vout	<= rc(r, c, pvout, ideal_vout, vout_min,vout_max);
      pvout	<= vout AFTER analog_time_delta;

END vg_amplifier_behavior;
