-- interface
ENTITY single_shot IS
     GENERIC (	 pulse_width: time := 50 ns;
		 t_delay    : time := 1 ns );
     PORT    (	 ss_in	: IN	 bit;
		 ss_out : BUFFER bit);

BEGIN
     ASSERT ( pulse_width >= 1 ns )
       REPORT "ERROR: pulse width of single shot must be >= 1 ns"
       SEVERITY failure;

     ASSERT ( t_delay >= 0 ps )
       REPORT "ERROR: time delay through single shot must be >= 0 ps"
       SEVERITY failure;

END single_shot;

-- body
ARCHITECTURE single_shot_behavior OF single_shot IS
  BEGIN
B1: BLOCK(ss_in = '1' AND ( NOT ss_in'STABLE) )
    BEGIN
       ss_out <= GUARDED '1' AFTER t_delay, '0' AFTER
				   (pulse_width + t_delay);
    END BLOCK b1;
END single_shot_behavior;
