USE work.analog_data_structure.all;

-- interface
ENTITY charge_pump IS
     GENERIC (	 imag	: real := 0.5;	 --current magnitude in ma
		 t_delay: time := 1 ns ); --time delay
     PORT    (	 up    : IN  bit;
		 down  : IN  bit;
		 iout  : BUFFER analog_signal := (val => 0.0, kind => i));

BEGIN
     ASSERT ( imag > 0.0 )
	REPORT "ERROR: charge pump current must be > 0.0."
	SEVERITY failure;

     ASSERT ( imag <= 10.0  )
	REPORT "ERROR: charge pump current must be < 10.0 ma."
	SEVERITY failure;
END charge_pump;

-- body
ARCHITECTURE charge_pump_behavior OF charge_pump IS

 BEGIN
   iout.kind  <= i;
   iout.val   <= imag AFTER t_delay WHEN (up = '1' AND down = '0') ELSE
		 0.0  AFTER t_delay WHEN (up = '0' AND down = '0') ELSE
		-imag AFTER t_delay WHEN (up = '0' AND down = '1') ELSE
		 0.0  AFTER t_delay;

END charge_pump_behavior;
