USE work.analog_data_structure.all;
USE work.analog_passive_behaviors.all;

-- interface
ENTITY vfo_filter  IS
    GENERIC (	r1: analog_component := ( val => 0.0, kind => res );
		c1: analog_component := ( val => 0.0, kind => cap );
		c2: analog_component := ( val => 0.0, kind => cap ) );
    PORT    (	iin : IN     analog_signal;
		vout: BUFFER analog_signal := (val => 2.0, kind => v));

BEGIN
    ASSERT  ( (r1.val > 0.0) AND (c1.val > 0.0) AND (c2.val > 0.0) )
       REPORT "ERROR: R and C's must be > 0.0"
       SEVERITY failure;
END vfo_filter;

-- body
ARCHITECTURE vfo_filter_behavior OF vfo_filter IS
	SIGNAL pvout, p2vout: analog_signal :=
	       ( val => 2.0, kind => v );

	SIGNAL piin: analog_signal :=
	       ( val => 0.0, kind => i );

    BEGIN
      vout   <= vfof(r1, c1, c2, piin, iin, p2vout, pvout);
      pvout  <= vout  AFTER analog_time_delta;
      piin   <= iin   AFTER analog_time_delta;
      p2vout <= pvout AFTER analog_time_delta;

END vfo_filter_behavior;
