USE work.analog_data_structure.all;
USE work.analog_active_behaviors.all;

-- interface
ENTITY vco  IS
     GENERIC (	 slope	  : real := 0.100; --ghz/v
		 intercept: real := 0.001; --ghz intercept
		 vhi	  : real := 1.0;   --vco cap high volt
		 vlo	  : real := 0.0 ); --vco cap low volt
     PORT    (	 vco_in  : IN	  analog_signal;
		 vco_out : BUFFER bit);

BEGIN
     ASSERT ( (vco_in.val * slope) > (-1.0 * intercept) )
	REPORT "ERROR: negative or zero vco frequency was requested"
	SEVERITY failure;

     ASSERT ( (vco_in.val * slope) < ( 800.0 * intercept) )
	REPORT "ERROR: too high a frequency was requested"
	SEVERITY failure;
END vco;

-- body
ARCHITECTURE vco_behavior OF vco IS
      SIGNAL  v_cap, v_flag : analog_signal :=
	      (val => 0.1E-3, kind => v);

    BEGIN
P1:   PROCESS(v_cap, vco_in)
      BEGIN
	 v_cap.val <=  vco_vcap (v_cap, vco_in, slope, intercept,
			 vhi, vlo, vco_out) AFTER analog_time_delta;
	 v_flag.val<=  vco_vcapf(v_cap, vco_in, slope, intercept,
				 vhi, vlo, vco_out);
      END PROCESS P1;
B1:   BLOCK( (v_flag.val >= vhi) OR (v_flag.val <= vlo) )
      BEGIN
	vco_out <= GUARDED
		'1' WHEN ((v_cap.val >= (vhi-vlo)/2.0) ) ELSE
		'0';

      END BLOCK B1;
END vco_behavior;
