USE work.tline_data_structure.all;

--interface
ENTITY main_network IS
END main_network;

--body
ARCHITECTURE main_network_sample OF main_network IS
      SIGNAL drv_logic_in   : bit := '0';
      SIGNAL rec1_logic_out : bit := '0';
      SIGNAL rec2_logic_out : bit := '0';
      SIGNAL term_in	    : bit := '0';
      SIGNAL drv_out : tline_node := ( volt    => 0.0, g0     => g_min,
				       volt_fe => 0.0, cur_fe => 0.0   );
      SIGNAL rec_1   : tline_node := ( volt    => 0.0, g0     => g_min,
				       volt_fe => 0.0, cur_fe => 0.0   );
      SIGNAL rec_2   : tline_node := ( volt    => 0.0, g0     => g_min,
				       volt_fe => 0.0, cur_fe => 0.0   );
      SIGNAL term    : tline_node := ( volt    => 0.0, g0     => g_min,
				       volt_fe => 0.0, cur_fe => 0.0   );
      COMPONENT  fet_od_drv
	     GENERIC ( gm      : real;
		       wl      : real;
		       vt      : real;
		       vgs     : real;
		       t_delay : time	    );
	     PORT    ( din : IN      bit;
		       dout: INOUT   tline_node);
      END COMPONENT;

      FOR ALL: fet_od_drv USE
	 ENTITY work.fet_od_drv(fet_od_drv_behavior);

      COMPONENT  receiver
	     GENERIC ( v_sw_hi	: real	    ;
		       v_sw_low : real	    ;
		       r_input	: real ;
		       t_delay	: time	     );
	     PORT    ( rec_in : INOUT	tline_node;
		       rec_out: BUFFER	bit	 );
      END COMPONENT;

      FOR ALL: receiver USE
	 ENTITY work.receiver(receiver_behavior);

      COMPONENT  linear_drv
	     GENERIC ( hi      : real	    ;
		       low     : real	    ;
		       d_kind  : sig_mode   ;
		       r_out   : real	    ;
		       t_delay : time	     );
	     PORT    ( drv_in : IN	bit;
		       drv_out: INOUT	tline_node);
      END COMPONENT;

      FOR ALL: linear_drv USE
	 ENTITY work.linear_drv(linear_drv_behavior);

      COMPONENT  tline
	     GENERIC ( r_char  : real ;
		       t_flight: time	    );
	     PORT   ( node1 : INOUT    tline_node;
		      node2 : INOUT   tline_node);
      END COMPONENT;

      FOR ALL: tline USE
	 ENTITY work.tline(tline_behavior);

  --instantiate
    BEGIN

      term_in	   <= '1' AFTER 100 ps;
      drv_logic_in <= '1' AFTER 200 ns, '0' AFTER 400 ns;

      D1: fet_od_drv  GENERIC MAP (  gm      =>   0.02,
				     wl      => 200.0 ,
				     vt      =>   1.0 ,
				     vgs     =>   5.0 ,
				     t_delay =>  1 ns	 )
		      PORT MAP	  (  drv_logic_in, drv_out );

      T0: tline       GENERIC MAP (  r_char   =>  0.08,
				     t_flight => 10 ns	 )
		      PORT MAP	  (  drv_out, term );

      V1: linear_drv  GENERIC MAP (  hi      =>  5.0  ,
				     low     =>  0.0  ,
				     d_kind  =>  v    ,
				     r_out   =>  0.200,
				     t_delay =>   100 ps )
		      PORT MAP	  (  term_in, term);

      T1: tline       GENERIC MAP (  r_char   =>  0.08,
				     t_flight =>  5 ns	 )
		      PORT MAP	  (  term, rec_1 );

      T2: tline       GENERIC MAP (  r_char   =>  0.05,
				     t_flight =>  2 ns	 )
		      PORT MAP	  (  term, rec_2 );

      R1: receiver    GENERIC MAP (  v_sw_hi  =>  2.5,
				     v_sw_low =>  2.5,
				     r_input  => 10.0,
				     t_delay  => 1 ns	 )
		      PORT MAP	  (  rec_1, rec1_logic_out );

      R2: receiver    GENERIC MAP (  v_sw_hi  =>  2.5,
				     v_sw_low =>  2.5,
				     r_input  => 10.0,
				     t_delay  => 1 ns	 )
		      PORT MAP	  (  rec_2, rec2_logic_out );
--
END main_network_sample;
