USE work.analog_data_structure.all;
--
ENTITY main_amp IS
END main_amp;
--
ARCHITECTURE main_amp_sample OF main_amp IS
    SIGNAL   vinp, vinn     : analog_signal := (val => 0.0, kind => v);
    SIGNAL   vout	    : analog_signal := (val => 0.0, kind => v);
    SIGNAL   aref	    : analog_reference_aref := (val => 1.2);
    SIGNAL   rec1_logic_out : bit := '0';

    COMPONENT amplifier
      GENERIC ( gain	 : analog_vv_gain;
		vcmo	 : analog_v_level;
		vout_min : analog_v_level;
		vout_max : analog_v_level;
		r	 : analog_component;
		c	 : analog_component);
      PORT   (	vinp	 : IN	  analog_signal;
		vinn	 : IN	  analog_signal;
		aref	 : IN	  analog_reference_aref;
		vout	 : BUFFER analog_signal );
    END COMPONENT;

    FOR ALL: amplifier USE
      ENTITY work.amplifier(amplifier_behavior);

    COMPONENT func_gen
      GENERIC ( amp	     : amplitude;
		offset	     : analog_v_level;
		freq	     : frequency);
      PORT   (	v_sine	     : OUT    analog_signal;
		v_sine2      : OUT    analog_signal);
    END COMPONENT;

    FOR ALL: func_gen USE
      ENTITY work.func_gen(func_gen_behavior);

    COMPONENT  receiver1
	   GENERIC ( v_sw_hi  : real	  ;
		     v_sw_low : real	  ;
		     t_delay  : time	   );
	   PORT    ( rec_in : IN      analog_signal;
		     rec_out: BUFFER  bit      );
    END COMPONENT;

    FOR ALL: receiver1 USE
       ENTITY work.receiver1(receiver1_behavior);

  --
  BEGIN
   --

   FG1 : func_gen GENERIC MAP ( (val => 0.1  ),
				(val => 3.0  ),
				(val => 0.010) )
		  PORT MAP    (vinp, vinn);

   AMP1: amplifier GENERIC MAP ( (val =>    4.0),
				 (val =>    4.0),
				 (val =>    3.0),
				 (val =>    5.0),
				 (val =>   1.0, kind => res),
				 (val =>  15.9, kind => cap) )
		   PORT MAP    ( vinp, vinn, aref, vout);


   R1: receiver1   GENERIC MAP (  v_sw_hi  =>  4.1,
				  v_sw_low =>  3.9,
				  t_delay  => 1 ns    )
		   PORT MAP    (  vout, rec1_logic_out );
--
END main_amp_sample;
