USE work.analog_data_structure.all;
--
ENTITY main_agc IS
END main_agc;
--
ARCHITECTURE main_agc_sample OF main_agc IS
    SIGNAL   vinp, vinn     : analog_signal := (val => 0.0, kind => v);
    SIGNAL   vout	    : analog_signal := (val => 0.0, kind => v);
    SIGNAL   vgain	    : analog_signal := (val => 0.0, kind => v);
    SIGNAL   vref	    : analog_signal := (val => 0.1, kind => v);
    SIGNAL   vcenter	    : analog_signal := (val => 4.0, kind => v);
    SIGNAL   iout	    : analog_signal := (val => 0.0, kind => i);

    COMPONENT vg_amplifier
      GENERIC ( slope	 : real;
		intcpt	 : analog_vv_gain;
		vcmo	 : analog_v_level;
		vout_min : analog_v_level;
		vout_max : analog_v_level;
		r	 : analog_component;
		c	 : analog_component);
      PORT   ( vinp   : IN     analog_signal;
	       vinn   : IN     analog_signal;
	       vgain  : IN     analog_signal;
	       vout   : BUFFER analog_signal);
    END COMPONENT;

    FOR ALL: vg_amplifier USE
	ENTITY work.vg_amplifier(vg_amplifier_behavior);

    COMPONENT  env
      GENERIC ( imag_sink   : real ;
		imag_source : real ;
		db_percent  : real ;
		t_delay     : time  );
      PORT   ( vin     : IN	analog_signal;
	       iout    : BUFFER analog_signal;
	       vref    : IN	analog_signal;
	       vcenter : IN	analog_signal);
    END COMPONENT;

	 FOR ALL: env USE
	    ENTITY work.env(env_behavior);

    COMPONENT func_gen
      GENERIC ( amp	     : amplitude;
		offset	     : analog_v_level;
		freq	     : frequency);
      PORT   (	v_sine	  : OUT    analog_signal;
		v_sine2   : OUT    analog_signal);
    END COMPONENT;

    FOR ALL: func_gen USE
	ENTITY work.func_gen(func_gen_behavior);

    COMPONENT cap_filter
      GENERIC ( c  : analog_component);
      PORT   (	iin	  : IN	   analog_signal;
		vout	  : BUFFER analog_signal);
    END COMPONENT;

    FOR ALL: cap_filter USE
	ENTITY work.cap_filter(cap_filter_behavior);
  --
  BEGIN
   --

   FG1 : func_gen     GENERIC MAP ( (val => 0.01 ),
				    (val => 3.0  ),
				    (val => 0.010) )
		      PORT MAP (vinp, vinn);

   vga1: vg_amplifier GENERIC MAP (	    100.0,   -- gain/v
				    (val => -200.0),
				    (val =>    4.0),
				    (val =>    3.0),
				    (val =>    5.0),
				    (val =>    1.0, kind => res),
				    (val =>   15.9, kind => cap) )
		      PORT MAP	  ( vinp, vinn, vgain, vout);

   ed1: env	      GENERIC MAP ( 0.5, 0.35, 0.2, 1 ns )
		      PORT MAP	  ( vout, iout, vref, vcenter);

   c1: cap_filter     GENERIC MAP ( (val =>  100.0, kind => cap)  )
		      PORT MAP	  ( iout, vgain);

--
END main_agc_sample;
