USE work.analog_data_structure.all;

-- interface
ENTITY main_pll IS
END main_pll;

-- body
ARCHITECTURE main_pll_sample OF main_pll IS
      SIGNAL fv_out : analog_signal := (val => 2.0, kind => v);
      SIGNAL iout   : analog_signal := (val => 0.0, kind => i);
      SIGNAL data_in: bit	    := '1';
      SIGNAL ss_out, inc, dec, vco_out	: bit := '0';

      COMPONENT single_shot
	   GENERIC( pulse_width : time;
		    t_delay	: time);
	   PORT   ( ss_in    : IN     bit;
		    ss_out   : BUFFER bit);
      END COMPONENT;

      FOR ALL: single_shot USE
	  ENTITY work.single_shot(single_shot_behavior);

      COMPONENT phase_freq
	   PORT   ( data_in : IN     bit;
		    vco_in  : IN     bit;
		    inc     : BUFFER bit;
		    dec     : BUFFER bit);
      END COMPONENT;

      FOR ALL: phase_freq USE
	  ENTITY work.phase_freq(phase_freq_structure);

      COMPONENT  charge_pump
	   GENERIC (  imag   : real;
		      t_delay: time );
	   PORT ( up   : IN  bit;
		  down : IN  bit;
		  iout : BUFFER analog_signal);
      END COMPONENT;

      FOR ALL: charge_pump USE
	  ENTITY work.charge_pump(charge_pump_behavior);

      COMPONENT  vfo_filter
	   GENERIC (  r1: analog_component;
		      c1: analog_component;
		      c2: analog_component );
	   PORT ( iin	: IN	 analog_signal;
		  vout	: BUFFER analog_signal);
      END COMPONENT;

      FOR ALL: vfo_filter USE
	  ENTITY work.vfo_filter(vfo_filter_behavior);

      COMPONENT  vco
	   GENERIC (  slope    : real;
		      intercept: real;
		      vhi      : real;
		      vlo      : real);
	   PORT ( vco_in : IN	  analog_signal;
		  vco_out: BUFFER bit);
      END COMPONENT;

      FOR ALL: vco USE
	  ENTITY work.vco(vco_behavior);

-- instantiate
    BEGIN
      data_in <= '1' AFTER 40 ns WHEN(data_in='0') ELSE
		 '0' AFTER 10 ns;

      ss1 : single_shot GENERIC MAP ( pulse_width => 25 ns,
				      t_delay	  => 1 ns)
			PORT MAP    ( data_in, ss_out);

      pd1: phase_freq	PORT MAP    ( ss_out, vco_out, inc, dec );

      cp1:  charge_pump GENERIC MAP ( imag => 0.5, t_delay => 1 ns )
			PORT MAP    ( inc, dec, iout);

      f1:  vfo_filter	GENERIC MAP ( r1 => (val =>  1.0, kind => res),
				      c1 => (val =>  5.0, kind => cap),
				      c2 => (val => 24.0, kind => cap) )
			PORT MAP    ( iout, fv_out);

      vco1: vco 	GENERIC MAP ( slope	=> 0.010,
				      intercept => 0.001,
				      vhi	=> 1.0	,
				      vlo	=> 0.0	 )
			PORT MAP    ( fv_out, vco_out);

END main_pll_sample;
