use work.sympack.all;
--
Package eislib_std is
type array4 is array (logic4'low to logic4'high) of logic4;
type logic4table is array (logic4'low to logic4'high) of array4;
type vector4 is array (natural range <>) of logic4;
type tpd_type is array (logic4'low to logic4'high) of positive;
type tpd_choice is array (logic4'low to logic4'high) of tpd_type;
--
-- functions and procedures
--
function is_bit_val(state:logic4) return boolean;
--
function t86(signal S1,S2:logic4;t1,t2,t3,t4:time) return time;
--
function longer(t1,t2:time) return time;
--
function pad(s1:string) return part_names;
--
function equ (s1,s2:part_names) return boolean;
--
function pick_tpd(part:part_names;choices:pm18) return positive;
--
function get_string(pn:part_names) return string;
--
function bv2i(b:bit_vector;sbv:boolean;sl,sh:natural) return integer;
--
-- logic operators definitions
--
constant e_xor: logic4table:=
  (('X','X','X','X'),
   ('X','0','1','X'),
   ('X','1','0','X'),
   ('X','X','X','Z'));
--
constant tpd_switch: tpd_choice:=
  ((5,5,5,5),
   (5,1,2,6),
   (5,3,4,7),
   (8,8,8,8));
--
end eislib_std;
--
Package body eislib_std is
function is_bit_val(state:logic4) return boolean is
begin
  if (state = '0') or (state = '1') then
    return TRUE;
  else
    return FALSE;
  end if;
end is_bit_val;
--
function t86(signal S1,S2:logic4;t1,t2,t3,t4:time) return time is
variable ptime: positive;
begin
  if not TPD_Handling then
    return 0 ns;
  else
    if S1'active then
      ptime := tpd_switch(S1)(S2);
    else
      ptime := tpd_switch(S2)(S1);
    end if;
    case ptime is
      when 1 => return t3; -- hl low
      when 2 => return t4; -- hl high
      when 3 => return t1; -- lh low
      when 4 => return t2; -- lh high
      when 5 => return longer(longer(t1,t2),longer(t3,t4)); -- worst case
      when 6 => return longer(t3,t4); -- hl low or high
      when 7 => return longer(t1,t2); -- lh low or high
      when 8 => return 0 ns; -- active signal is not driving;
      when others => null;
    end case;
  end if;
end t86;
--
function longer(t1,t2:time) return time is
begin
  if (t1 > t2) then
    return t1;
  else
    return t2;
  end if;
end longer;
--
function pad(s1:string) return part_names is
variable s2:part_names;
begin
  for i in 1 to s1'length loop
    s2(i) := s1(i);
  end loop;
  for i in s1'length+1 to 24 loop
    s2(i) := ' ';
  end loop;
  return s2;
end pad;
--
function bv2i(b:bit_vector;sbv:boolean;sl,sh:natural) return integer is
-- sbv is true if the vector has a sign bit in the most significant
-- position.
-- sl is the least significant bit index.
-- sh is the most significant bit index.
variable int,j:integer:=0;
begin
  j := sh;
  if sbv then
    j := j-1;
  end if;
  for i in j downto sl loop
    int := int*2;
    if b(i) = '1' then
      int := int + 1;
    end if;
  end loop;
  if sbv and b(sh) = '1' then
    int := -int;
  end if;
  return int;
end bv2i;
--
function get_string(pn:part_names) return string is
variable s1:string (1 to 24);
begin
  for i in 1 to pn'length loop
    if (pn(i) = ' ') then
      s1(i) := NUL;
      return s1;
    else
      s1(i) := pn(i);
    end if;
  end loop;
  return s1;
end get_string;
--
function equ (s1,s2:part_names) return boolean is
begin
  for i in 1 to 24 loop
    if s1(i) /= s2(i) then
      return FALSE;
    end if;
  end loop;
  return TRUE;
end equ;
--
--
function pick_tpd(part:part_names;choices:pm18) return positive is
variable s1:string (1 to 24);
begin
  for i in 1 to choices'length loop
    if equ (part,choices(i)) then
      return (i*4) - 3;
    end if;
  end loop;
  s1 := get_string(part);
  assert false
  report "specified part " & s1 & " is not supported"
  severity error;
end pick_tpd;
--
end eislib_std;
