--
-- Figure 7.2.  Structural Full-Adder Model
-- Taken from "Applications of VHDL to Circuit Design", page 200
--
-- Ken Scott
-- January 29, 1991
--
-- Modifications:
-- 1) Made the example self-contained.
-- 2) Added the gates package components so simulation possible
-- 

package gates_comp is
  component xor2 port( a,b : in bit; z : out bit ); 
  end component;
  component and2 port( a,b : in bit; z : out bit ); 
  end component;
  component or2 port( a,b : in bit; z : out bit ); 
  end component;
end gates_comp;

  entity xor2 is port( a,b : in bit; z : out bit ); 
  end;
  architecture sim of xor2 is
  begin
    z <= a xor b;
  end;
  entity and2 is port( a,b : in bit; z : out bit ); 
  end;
  architecture sim of and2 is
  begin
    z <= a and b;
  end;
  entity or2 is port( a,b : in bit; z : out bit ); 
  end;
  architecture sim of or2 is
  begin
    z <= a or b;
  end;

entity full_adder is
  port(
    a,b,cin : in bit;
    sum,cout : out bit );
end full_adder;

architecture structural of full_adder is
  use work.gates_comp.all;
  signal s1, s2, s3 : bit;
begin
  c1: xor2 port map( a ,b  ,s1 );
  c2: xor2 port map( s1,cin,sum );
  c3: and2 port map( s1,cin,s2 );
  c4: and2 port map( a ,b  ,s3 );
  c5: or2  port map( s2,s3 ,cout );
end structural; 
--
