--
-- Figure 7.5.  Multiple Drivers Driving Signal
-- Taken from "Applications of VHDL to Circuit Design", page 205
--
-- Ken Scott
-- January 29, 1991
--
-- Modifications:
-- 1) Added a package defining mvl and WIRE_X.  Also another
--    package defining functions.
-- 

package mvl_pack is
  type mvl is ( '0', '1', 'x', 'z' );
  type mvl_vector is array( natural range <> ) of mvl;
  function WIRE_X ( s : mvl_vector ) return mvl;
end mvl_pack;

package body mvl_pack is
  function WIRE_X ( s : mvl_vector ) return mvl is
    variable result : mvl := 'z';
  begin
    for I in s'range loop
      case s(i) is
      when '0' | '1'=> if (result /= 'z') then
                         assert (false) report "bus error";
                         return 'x';
                       else
                         result := s(i);
                       end if;
      when 'x'      => return 'x';
      when 'z'      => next;
      end case;
    end loop;
    return result;
  end;
end mvl_pack;

use work.mvl_pack.all;
package func_pack is
  function some_func return mvl;
  function some_other_func return mvl;
end func_pack;

package body func_pack is
  function some_func return mvl is
  begin
    return '0';  -- doesn't matter.
  end;
  function some_other_func return mvl is
  begin
    return '1';  -- doesn't matter.
  end;
end func_pack;

use work.mvl_pack.all;
use work.func_pack.all;
entity test is
  port( flag : out WIRE_X mvl := '0' );
end test;

architecture twodrivers of test is
begin
  p1: flag <= some_func;
  p2: flag <= some_other_func;
end twodrivers; 
--
