--
-- Figure 7.10.  Using VHDL Registers instead of Variables
-- Taken from "Applications of VHDL to Circuit Design", page 210
--
-- Ken Scott
-- January 29, 1991
--
-- Modifications:
-- 1) Making the example self-contained.
-- 

package outer is
  type int_array is array( natural range <> ) of integer;
  function resolved( s : in int_array ) return integer;
end outer;

package body outer is
  function resolved( s : in int_array ) return integer is
  begin
    if (s'length > 1) then
      if (now = 0 ns) then
        return s(s'left);
      end if;
      assert false report "Bus contention on signal count.";
      return 0;
    end if;
    return s(s'left);
  end;
end outer;
--

use work.outer.all;
entity counter is
  port( 
    clk,reset : in  bit;
    data  : out integer );
end counter;

architecture behavioral3 of counter is
  signal count : resolved integer register := 0;
begin
  process( reset )
  begin
    if (reset = '1') then
      count <= 0, null after 1 fs;
    else
      count <= null;
    end if;
  end process;
  process( clk )
  begin
    if (reset = '0') and (clk = '1') then
      count <= count + 1, null after 1 fs;
    else
      count <= null;
    end if;
  end process;

  data <= count;
end behavioral3;
--


