--
-- Figure 7.14.  Using Registers to Model the Behavior of a Bus
-- Taken from "Applications of VHDL to Circuit Design", page 216
--
-- Ken Scott
-- January 29, 1991
--
-- Modifications:
-- 1) Making the example self-contained.
-- 

package resolve_bit is
  function bus_resolve( s : bit_vector ) return bit;
  subtype rbit is bus_resolve bit;
  type rbit_vector is array( natural range <> ) of rbit;
  subtype rbit4 is rbit_vector( 3 downto 0 );
end resolve_bit;

package body resolve_bit is
  function bus_resolve( s : bit_vector ) return bit is
  begin
    if (s'length = 0) then
      assert (false) report "zero sources driving bus";
      return '0';
    elsif (s'length /= 1) and (now /= 0 ns) then
      assert (false) report "multiple sources driving bus";
      return '0';
    else
      return s(s'left);
    end if;
  end bus_resolve;
end resolve_bit;
--


use  work.resolve_bit.all;
entity micro_p is
  port(
    control_bus : in  bit;
    bus_sig: inout rbit4 );
end micro_p;

architecture example of micro_p is
begin
  -- Drives bus to all zero's when control signal is '0'.
  driver0:
  block(control_bus = '0') 
    signal bus_sig : rbit4 bus;
  begin
    bus_sig <= guarded b"0000";
  end block driver0;

  -- Drives bus to all one's when control signal is '1'.
  driver1:
  block(control_bus = '1')
    signal bus_sig : rbit4 bus;
  begin
    bus_sig <= guarded b"1111";
  end block driver1;
end example;
--


